/************************************************************************/
/*                                                                      */
/*		PmodJSTK2.cpp  --	PmodJSTK2 Command and Interface Functions	*/
/*                                                                      */
/************************************************************************/
/*	Author: 	Michael T. Alexander								    */
/*	Copyright:	2016, Digilent, Inc.									*/
/************************************************************************/
/*  File Description:                                                   */
/*                                                                      */
/*	This file contains the implementation of functions used for			*/
/*	accessing the PmodJSTK2.											*/
/*                                                                      */
/************************************************************************/
/*  Revision History:													*/
/*                                                                      */
/*	04/01/2016(MichaelA): created										*/
/*                                                                      */
/************************************************************************/

/* ------------------------------------------------------------ */
/*					Include File Definitions					*/
/* ------------------------------------------------------------ */

#include <windows.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "dpcdecl.h"
#include "dspi.h"

#include "PmodJSTK2_PC.h"

/* ------------------------------------------------------------ */
/*					Local Type Definitions						*/
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
/*					 Global Variables							*/
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
/*					Local Variables								*/
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
/*                  Procedure Definitions                       */
/* ------------------------------------------------------------ */
/***    FGetJstk
**
**  Parameters:
**      hifJstk		- open handle to DSPI port configured to communicate with joystick
**      pjstkcmdSnd - pointer to joystick command structure containing data to send
**		pjstkpktRcv	- pointer to struct to receive joystick packet
**		cbSnd		- number of bytes to transfer
**					- if greater than 5 then a check will be performed
**					  to see if the EXTPKT bit is set and if it's not
**					  then an error will be returned
**
**  Return Values:
**      fTrue for success, fFalse otherwise
**
**  Errors:
**
**  Description:
**      This function transfers a command packet to the PmodJSTK2
**		using the DSPI port referenced by hifJstk and receives 
**		cbSnd bytes from the device. If NULL isn't specified for
**		pjstkpktRcv then any received bytes are stored in the structure
**		pointed to by pjstkpktRcv. Please note that it is up to the caller
**		to specify an appropriate number of bytes for the command.
**		Also note that if the byte count is >= 6 then this function
**		will check to see if the EXTPKT bit is set in the fsButtons byte,
**		and if it's not present then an error will be returned.
*/
BOOL	
FGetJstk(HIF hifJstk, JSTKCMD* pjstkcmdSnd, JSTKPKT* pjstkpktRcv, BYTE cbSnd) {

	BYTE	rgbSnd[cbSndMax];
	BYTE	rgbRcv[cbSndMax];

	if ( hifInvalid == hifJstk ) {
		return fFalse;
	}

	if ( NULL == pjstkcmdSnd ) {
		return fFalse;
	}

	if ( cbSndMax < cbSnd ) {
		return fFalse;
	}

	memset(rgbSnd, 0, cbSndMax);
	memset(rgbRcv, 0, cbSndMax);
	memcpy(rgbSnd, pjstkcmdSnd, sizeof(JSTKCMD));
	if ( ! DspiPut(hifJstk, fFalse, fTrue, rgbSnd, rgbRcv, cbSnd, fFalse) ) {
		return fFalse;
	}

	if (( 5 < cbSnd ) && ( 0 == (rgbRcv[4] & bitExtpkt) )) {
		return fFalse;
	}
	
	if ( NULL != pjstkpktRcv ) {
		memcpy(pjstkpktRcv, rgbRcv, sizeof(JSTKPKT));
	}

	return fTrue;
}

/* ------------------------------------------------------------ */
/***    FGetJstkCAL
**
**  Parameters:
**      hifJstk		- open handle to DSPI port configured to communicate with joystick
**      pjstkcalRcv - pointer to struct to receive joystick calibration variables
**
**  Return Values:
**      fTrue for success, fFalse otherwise
**
**  Errors:
**
**  Description:
**      This function uses the DSPI port referenced by hifJstk to
**		retrieve the calibration constants from the PmodJSTK2.
*/
BOOL	
FGetJstkCAL(HIF hifJstk, JSTKCAL* pjstkcalRcv) {

	JSTKCMD	jstkcmd;
	JSTKPKT	jstkpkt;
	JSTKCAL	jstkcal;

	if ( hifInvalid == hifJstk ) {
		return fFalse;
	}

	if ( NULL == pjstkcalRcv ) {
		return fFalse;
	}

	memset(&jstkcmd, 0, sizeof(JSTKCMD));
	jstkcmd.cmd = cmdGetCalXMin;
	if ( ! FGetJstk(hifJstk, &jstkcmd, &jstkpkt, 7) ) {
		return fFalse;
	}
	jstkcal.smpXMin = jstkpkt.calvar;

	jstkcmd.cmd = cmdGetCalXMax;
	if ( ! FGetJstk(hifJstk, &jstkcmd, &jstkpkt, 7) ) {
		return fFalse;
	}
	jstkcal.smpXMax = jstkpkt.calvar;

	jstkcmd.cmd = cmdGetCalYMin;
	if ( ! FGetJstk(hifJstk, &jstkcmd, &jstkpkt, 7) ) {
		return fFalse;
	}
	jstkcal.smpYMin = jstkpkt.calvar;

	jstkcmd.cmd = cmdGetCalYMax;
	if ( ! FGetJstk(hifJstk, &jstkcmd, &jstkpkt, 7) ) {
		return fFalse;
	}
	jstkcal.smpYMax = jstkpkt.calvar;

	jstkcmd.cmd = cmdGetCalXCenMin;
	if ( ! FGetJstk(hifJstk, &jstkcmd, &jstkpkt, 7) ) {
		return fFalse;
	}
	jstkcal.smpXCenterMin = jstkpkt.calvar;

	jstkcmd.cmd = cmdGetCalXCenMax;
	if ( ! FGetJstk(hifJstk, &jstkcmd, &jstkpkt, 7) ) {
		return fFalse;
	}
	jstkcal.smpXCenterMax = jstkpkt.calvar;

	jstkcmd.cmd = cmdGetCalYCenMin;
	if ( ! FGetJstk(hifJstk, &jstkcmd, &jstkpkt, 7) ) {
		return fFalse;
	}
	jstkcal.smpYCenterMin = jstkpkt.calvar;

	jstkcmd.cmd = cmdGetCalYCenMax;
	if ( ! FGetJstk(hifJstk, &jstkcmd, &jstkpkt, 7) ) {
		return fFalse;
	}
	jstkcal.smpYCenterMax = jstkpkt.calvar;

	memcpy(pjstkcalRcv, &jstkcal, sizeof(JSTKCAL));

	return fTrue;
}

BOOL	
FGetJstkFWVER(HIF hifJstk, WORD* pfwverRcv) {

	JSTKCMD	jstkcmd;
	JSTKPKT	jstkpkt;

	/* Attempt to retrieve the firmware version from the
	** device.
	*/
	memset(&jstkcmd, 0, sizeof(JSTKCMD));
	jstkcmd.cmd = cmdGetFirmwareVer;
	if ( ! FGetJstk(hifJstk, &jstkcmd, &jstkpkt, 7) ) {
		return fFalse;
	}

	if ( NULL != pfwverRcv ) {
		*pfwverRcv = jstkpkt.fwver;
	}

	return fTrue;
}

BOOL	
FCalibrateJstk(HIF hifJstk) {

	JSTKCMD	jstkcmd;
	JSTKPKT	jstkpkt;

	if ( hifInvalid == hifJstk ) {
		return fFalse;
	}

	/* Send the calibrate command.
	*/
	memset(&jstkcmd, 0, sizeof(JSTKCMD));
	jstkcmd.cmd = cmdCalibrate;
	if ( ! FGetJstk(hifJstk, &jstkcmd, &jstkpkt, 5) ) {
		return fFalse;
	}

	/* Give the joystick some time to enter calibration
	** mode before we start polling the status register.
	*/
	Sleep(10);

	/* Read the joystick status register.
	*/
	jstkcmd.cmd = cmdGetStatus;
	if ( ! FGetJstk(hifJstk, &jstkcmd, &jstkpkt, 6) ) {
		return fFalse;
	}

	while ( jstkpkt.sts & bitCalibrating ) {
		
		/* Kill a little time before reading the status register
		** again.
		*/
		Sleep(10);

		/* Read the joystick status register.
		*/
		jstkcmd.cmd = cmdGetStatus;
		if ( ! FGetJstk(hifJstk, &jstkcmd, &jstkpkt, 6) ) {
			return fFalse;
		}
	}

	/* Calibration completed. Was it successful?
	*/
	if ( 0 == (jstkpkt.sts & bitLastCal) ) {
		return fFalse;
	}

	return fTrue;
}