/************************************************************************/
/*                                                                      */
/*		PmodJSTK2.cpp  --	PmodJSTK2 Command and Interface Functions	*/
/*                                                                      */
/************************************************************************/
/*	Author: 	James K. Colvin										    */
/*	Copyright:	2016, Digilent, Inc.									*/
/************************************************************************/
/*  File Description:                                                   */
/*                                                                      */
/*	This file contains the implementation of functions used for			*/
/*	accessing the PmodJSTK2.											*/
/*                                                                      */
/************************************************************************/
/*  Revision History:													*/
/*                                                                      */
/*	04/01/2016(MichaelA): created PC based setup						*/
/*	04/25/2016(JamesC): modified PC setup for a Digilent MCU			*/
/*                                                                      */
/************************************************************************/

/* ------------------------------------------------------------ */
/*					Include File Definitions					*/
/* ------------------------------------------------------------ */

#include <DSPI.h>

#include <PmodJSTK2.h>

/* ------------------------------------------------------------ */
/*					Local Type Definitions						*/
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
/*					 Global Variables							*/
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
/*					Local Variables								*/
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
/*                  Procedure Definitions                       */
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
/*	JSTK2::JSTK2()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Constructs the JSTK2 class
*/
JSTK2::JSTK2()
{
	
}

/* ------------------------------------------------------------ */
/*	JSTK2::begin(int SpiPort, bool userDelay)
**
**	Parameters:
**		SpiPort		-	which DSPI controller should be used
**						default value of 0
**		userDelay	-	a flag saying if users should implement their
**						own delays, default value of false
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Chooses and constructs the DSPI controller and then calls
**		JSTK2HostInit() to intialize the SPI hardware
*/
void JSTK2::begin(int SpiPort, bool userDelay)
{
	switch(SpiPort){
#if (NUM_DSPI_PORTS > 3)
		case 3:
			pdspi = new DSPI3();
			m_SSPin = PIN_DSPI3_SS;	// default SS pin for DSPI3
			break;
#endif

#if (NUM_DSPI_PORTS > 2)
		case 2:
			pdspi = new DSPI2();
			m_SSPin = PIN_DSPI2_SS;	// default SS pin for DSPI2
			break;
#endif

#if (NUM_DSPI_PORTS > 1)
		case 1:
			pdspi = new DSPI1();
			m_SSPin = PIN_DSPI1_SS;	// default SS pin for DSPI1
			break;
#endif

		default:
			pdspi = new DSPI0();
			#if defined (_BOARD_WF32_)
			m_SSPin = 10;	//jumper JP3 on PWM side
			#else	//all other boards
			m_SSPin = PIN_DSPI0_SS;	// default SS pin for DSPI0, this already uses the JP3 on the
									//pwm side for other similar form factor boards
			#endif
	} //#include <DSPI.h>
		//	DSPI0 dspi
		//setup
		//dspi.begin();
		//dspi.setSpeed(1000000);
		//loop
		//dspi.setSelect(LOW);
		//dspi.transfer(data);
		//dspi.setSelect(HIGH);
	JSTK2HostInit();	//initiate the SPI hardware
	//set the library flag for post SPI delays
	if(userDelay == true){
		manualSpiDelay = true;
	}
	else{
		manualSpiDelay = false;
	}
}


/* ------------------------------------------------------------ */
/*	JSTK2::JSTK2HostInit
**
**	Parameters:
**		
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Initiate the SPI controller once it has been chosen.
*/
void JSTK2::JSTK2HostInit()
{
	pdspi->begin(m_SSPin);
	pdspi->setMode(DSPI_MODE0);	//presumably this mode is correct
	pdspi->setSpeed(1000000);	//1 MHz SPI clock
}

/**
	Starting with the set of legacy commands that need to be supported
**/

/* ------------------------------------------------------------ */
/*	JSTK2::setLeds(uint8_t ledSt)
**
**	Parameters:
**		ledSt		-	a 1 or a 0 to turn the green LED on or off, respectively
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Turn the green LED on or off, and set the red and blue LEDs to off
*/
void JSTK2::setLeds(uint8_t ledSt)
{
	if(ledSt == 1 || ledSt == 3){
		setLedRGB(0, 255, 0);	//turn the green LED on full bright
	}
	else{
		setLedRGB(0, 0, 0);		//turn the green LED off full bright
	}
}

/* ------------------------------------------------------------ */
/*	JSTK2::getX()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Retrieve the X axis value and return it to the user
*/
uint16_t JSTK2::getX()
{
	uint16_t xPos;
	uint8_t recv[5];
	getAll(0,recv);

	xPos = recv[0];
	xPos |= (recv[1] << 8);

	return xPos;
}

/* ------------------------------------------------------------ */
/*	JSTK2::getY()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Retrieve the Y axis value and return it to the user
*/
uint16_t JSTK2::getY()
{
	uint16_t yPos;
	uint8_t recv[5];
	getAll(0,recv);

	yPos = recv[2];
	yPos |= (recv[3] << 8);

	return yPos;
}

/* ------------------------------------------------------------ */
/*	JSTK2::getBtn()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Retrieve the state of the buttons and return it to the user
*/
uint8_t JSTK2::getBtn()
{
	uint8_t recv[5];
	getAll(0,recv);

	return recv[4];
}

/* ------------------------------------------------------------ */
/*	JSTK2::getAll(uint8_t ledSt, uint8_t *recv)
**
**	Parameters:
**		ledSt		-	a 1 or a 0 to turn the green LED on or off, respectively
**		*recv		-	user array to receive each of the 5 standard bytes
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Turn the green LED on or off and set the red and blue LEDs to off
**		Then retrieve the 5 standard bytes and assign them to the pointer
**		used by the getX, getY, and getBtn functions.
*/
void JSTK2::getAll(uint8_t ledSt, uint8_t *recv)
{
	setLeds(ledSt);
	*recv++ = (uint8_t)jstkPkt.smpX;
	*recv++ = jstkPkt.smpX>>8;
	*recv++ = (uint8_t)jstkPkt.smpY;
	*recv++ = jstkPkt.smpY>>8;
	*recv = jstkPkt.fsBtns;
}

/**
	Now moving on to the new PmodJSTK2 functions
**/

/* ------------------------------------------------------------ */
/*	JSTK2::setLedOn()
**
**	Parameters:
**		
**
**	Return Value:
**		
**
**	Errors:
**		none
**
**	Description:
**		
*/
void JSTK2::setLedOn()
{
	setLeds(1);
}

/* ------------------------------------------------------------ */
/*	JSTK2::setLedOff()
**
**	Parameters:
**		
**
**	Return Value:
**		
**
**	Errors:
**		none
**
**	Description:
**		
*/
void JSTK2::setLedOff()
{
	setLeds(0);
}

/* ------------------------------------------------------------ */
/*	JSTK2::setLedRGB(uint8_t red, uint8_t green, uint8_t blue)
**
**	Parameters:
**		uint8_t red			- duty cycle of the red LED
**		uint8_t green		- duty cycle of the green LED
**		uint8_t blue		- duty cycle of the blue LED
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		set the duty cycles of the red, green, and blue LEDs on the RGB LED
*/
void JSTK2::setLedRGB(uint8_t red, uint8_t green, uint8_t blue)
{
	setCommand(cmdSetLedRGB, red, green, blue);
}

/* ------------------------------------------------------------ */
/*	JSTK2::setInversion(uint8_t xInversion, uint8_t yInversion)
**
**	Parameters:
**		uint8_t xInversion
**		uint8_t yInversion
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		set or disable the inversion of the x and y axis positions (not the raw 10-bit values)
**		Identical to the setInversionXY function below this one
**		
*/
void JSTK2::setInversion(uint8_t xInversion, uint8_t yInversion)
{
	if(xInversion == 1 && yInversion == 0){
		setCommand(cmdSetInversionX);
	}
	else if(xInversion == 0 && yInversion == 1){
		setCommand(cmdSetInversionY);
	}
	else if(xInversion == 1 && yInversion == 1){
		setCommand(cmdSetInversionX|cmdSetInversionY);
	}
	else{
		setCommand(cmdSetInversionOff);
	}
}

/* ------------------------------------------------------------ */
/*	JSTK2::setInversionXY(uint8_t xInversion, uint8_t yInversion)
**
**	Parameters:
**		uint8_t xInversion
**		uint8_t yInversion
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		set or disable the inversion of the x and y axis positions (not the raw 10-bit values)
**		
*/
void JSTK2::setInversionXY(uint8_t xInversion, uint8_t yInversion)
{
	if(xInversion == 1 && yInversion == 0){
		setCommand(cmdSetInversionX);
	}
	else if(xInversion == 0 && yInversion == 1){
		setCommand(cmdSetInversionY);
	}
	else if(xInversion == 1 && yInversion == 1){
		setCommand(cmdSetInversionX|cmdSetInversionY);
	}
	else{
		setCommand(cmdSetInversionOff);
	}
}

/* ------------------------------------------------------------ */
/*	JSTK2::setInversionX(uint8_t xInversion)
**
**	Parameters:
**		uint8_t xInversion		- set X inversion to be true (1) or false (0)
**
**	Return Value:
**		
**
**	Errors:
**		none
**
**	Description:
**		Set the X inversion to be true (1) or false (0) and set the y inversion to be false
*/
void JSTK2::setInversionX(uint8_t xInversion)
{
	setInversionXY(xInversion, 0);
}

/* ------------------------------------------------------------ */
/*	JSTK2::setInversionY(uint8_t yInversion)
**
**	Parameters:
**		uint8_t yInversion		- set Y inversion to be true (1) or false (0)
**
**	Return Value:
**		
**
**	Errors:
**		none
**
**	Description:
**		Set the Y inversion to be true (1) or false (0) and set the x inversion to be false
*/
void JSTK2::setInversionY(uint8_t yInversion)
{
	setInversionXY(0, yInversion);
}

/* ------------------------------------------------------------ */
/*	JSTK2::setInversionOff()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Disable the inversion for both axes
*/
void JSTK2::setInversionOff()
{
	setInversionXY(0, 0);
}

/* ------------------------------------------------------------ */
/*	JSTK2::getPosition()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		
*/
void JSTK2::getPosition()
{
	uint8_t positionValues[2];
	receiveCommand(cmdGetPosition, 2, (uint8_t *)positionValues);
	jstkPkt.posX = positionValues[0];
	jstkPkt.posY = positionValues[1];
}

/* ------------------------------------------------------------ */
/*	JSTK2::getStatus()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Collect the status value from the getStatus command and store
**		the result in the global struct variable declared in the .h file.
**		The 5 standard bytes are collected and stored in the struct
**		variables via the receiveCommand function.
**		
*/
void JSTK2::getStatus()
{
	uint8_t statusValue[1];
	receiveCommand(cmdGetStatus, 1, (uint8_t *)statusValue);
	jstkPkt.sts = statusValue[0];
}

/* ------------------------------------------------------------ */
/*	JSTK2::getFirmwareVer()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Collect the firmware value from the getFirmwareVer command and store
**		the result in the global struct variable declared in the .h file.
**		The 5 standard bytes are collected and stored in the struct
**		variables via the receiveCommand function.
**		
*/
void JSTK2::getFirmwareVer()
{
	uint8_t firmwareValue[2];
	receiveCommand(cmdGetFirmwareVer, 2, (uint8_t *)firmwareValue);
	jstkPkt.fwver = ((uint16_t)firmwareValue[1] << 8) | firmwareValue[0];
}

/* ------------------------------------------------------------ */
/*	JSTK2::getCalXMin()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function collects the current calibration value from 
**		RAM and stores the result in the global struct variable 
**		declared in the .h file.
**		The 5 standard bytes are collected and stored in the struct
**		variables via the receiveCommand function.
*/
void JSTK2::getCalXMin()
{
	uint8_t calibrationValue[2];
	receiveCommand(cmdGetCalXMin, 2, (uint8_t *)calibrationValue);
	jstkPkt.calvar = ((uint16_t)calibrationValue[1] << 8) | calibrationValue[0];
	jstkCal.smpXMin = ((uint16_t)calibrationValue[1] << 8) | calibrationValue[0];
}

/* ------------------------------------------------------------ */
/*	JSTK2::getCalXMax()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function collects the current calibration value from 
**		RAM and stores the result in the global struct variable 
**		declared in the .h file.
**		The 5 standard bytes are collected and stored in the struct
**		variables via the receiveCommand function.
*/
void JSTK2::getCalXMax()
{
	uint8_t calibrationValue[2];
	receiveCommand(cmdGetCalXMax, 2, (uint8_t *)calibrationValue);
	jstkPkt.calvar = ((uint16_t)calibrationValue[1] << 8) | calibrationValue[0];
	jstkCal.smpXMax = ((uint16_t)calibrationValue[1] << 8) | calibrationValue[0];
}

/* ------------------------------------------------------------ */
/*	JSTK2::getCalYMin()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function collects the current calibration value from 
**		RAM and stores the result in the global struct variable 
**		declared in the .h file.
**		The 5 standard bytes are collected and stored in the struct
**		variables via the receiveCommand function.
*/
void JSTK2::getCalYMin()
{
	uint8_t calibrationValue[2];
	receiveCommand(cmdGetCalYMin, 2, (uint8_t *)calibrationValue);
	jstkPkt.calvar = ((uint16_t)calibrationValue[1] << 8) | calibrationValue[0];
	jstkCal.smpYMin = ((uint16_t)calibrationValue[1] << 8) | calibrationValue[0];
}

/* ------------------------------------------------------------ */
/*	JSTK2::getCalYMax()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function collects the current calibration value from 
**		RAM and stores the result in the global struct variable 
**		declared in the .h file.
**		The 5 standard bytes are collected and stored in the struct
**		variables via the receiveCommand function.
*/
void JSTK2::getCalYMax()
{
	uint8_t calibrationValue[2];
	receiveCommand(cmdGetCalYMax, 2, (uint8_t *)calibrationValue);
	jstkPkt.calvar = ((uint16_t)calibrationValue[1] << 8) | calibrationValue[0];
	jstkCal.smpYMax = ((uint16_t)calibrationValue[1] << 8) | calibrationValue[0];
}

/* ------------------------------------------------------------ */
/*	JSTK2::getCalXCenMin()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function collects the current calibration value from 
**		RAM and stores the result in the global struct variable 
**		declared in the .h file.
**		The 5 standard bytes are collected and stored in the struct
**		variables via the receiveCommand function.
*/
void JSTK2::getCalXCenMin()
{
	uint8_t calibrationValue[2];
	receiveCommand(cmdGetCalXCenMin, 2, (uint8_t *)calibrationValue);
	jstkPkt.calvar = ((uint16_t)calibrationValue[1] << 8) | calibrationValue[0];
	jstkCal.smpXCenterMin = ((uint16_t)calibrationValue[1] << 8) | calibrationValue[0];
}

/* ------------------------------------------------------------ */
/*	JSTK2::getCalXCenMax()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function collects the current calibration value from 
**		RAM and stores the result in the global struct variable 
**		declared in the .h file.
**		The 5 standard bytes are collected and stored in the struct
**		variables via the receiveCommand function.
*/
void JSTK2::getCalXCenMax()
{
	uint8_t calibrationValue[2];
	receiveCommand(cmdGetCalXCenMax, 2, (uint8_t *)calibrationValue);
	jstkPkt.calvar = ((uint16_t)calibrationValue[1] << 8) | calibrationValue[0];
	jstkCal.smpXCenterMax = ((uint16_t)calibrationValue[1] << 8) | calibrationValue[0];
}

/* ------------------------------------------------------------ */
/*	JSTK2::getCalYCenMin()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function collects the current calibration value from 
**		RAM and stores the result in the global struct variable 
**		declared in the .h file.
**		The 5 standard bytes are collected and stored in the struct
**		variables via the receiveCommand function.
*/
void JSTK2::getCalYCenMin()
{
	uint8_t calibrationValue[2];
	receiveCommand(cmdGetCalYCenMin, 2, (uint8_t *)calibrationValue);
	jstkPkt.calvar = ((uint16_t)calibrationValue[1] << 8) | calibrationValue[0];
	jstkCal.smpYCenterMin = ((uint16_t)calibrationValue[1] << 8) | calibrationValue[0];
}

/* ------------------------------------------------------------ */
/*	JSTK2::getCalYCenMax()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function collects the current calibration value from 
**		RAM and stores the result in the global struct variable 
**		declared in the .h file.
**		The 5 standard bytes are collected and stored in the struct
**		variables via the receiveCommand function.
*/
void JSTK2::getCalYCenMax()
{
	uint8_t calibrationValue[2];
	receiveCommand(cmdGetCalYCenMax, 2, (uint8_t *)calibrationValue);
	jstkPkt.calvar = ((uint16_t)calibrationValue[1] << 8) | calibrationValue[0];
	jstkCal.smpYCenterMax = ((uint16_t)calibrationValue[1] << 8) | calibrationValue[0];
}

/* ------------------------------------------------------------ */
/*	JSTK2::calibrate()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Once chip select goes high after the setCommand is complete, 
**		the PIC16 present on the PmodJSTK2 for the next 10 seconds
**		looks to see if the measured ADC values remain unchanged for
**		an entire second, presuming (and checking) that all axes exceed
**		an internally predefined constant.
**		
**		During the calibration sequence, the blue LED will be flashing
**		
**		If calibration is unsuccessful (module not still for an entire second
**		or does not exceed internally set values) the blue LED will stop flashing
**		and the red LED will flash 2 times and the LASTCAL and CALIBRATING bits 
**		of the status register will be cleared.
**		
**		If calibration is successful (the measured adc values for each axis have
**		not changed for an entire second, so the module presumes that this is the
**		center position of the joystick), the blue LED will stop flashing and the
**		green LED will flash 2 times and the LASTCAL bit in the status register
**		will be set and the CALIBRATING bit of the status register will be cleared.
**		All of the most recent calculated calibration constants will be stored in 
**		the RAM of the PIC16.
**		
**		During the calibration sequence, the user may poll the status of the device
**		by reading the status register through the getStatus() function and observe
**		the CALIBRATING bit (bit 7).
**		
**		All of the calibration constants that are calculated are the 10 bit values
**		which are used to then calculate the 8-bit position values (as opposed to
**		raw ADC values) of the joystick itself.
*/
void JSTK2::calibrate()
{
	//send the command
	setCommand(cmdCalibrate);
	//Users should note that while the calibration sequence is taking place
	//(indicated by the flashing blue LED), the PmodJSTK2 will no accept
	//any new SPI commands. However, users may pool the status register via
	//the getStatus() function and look to see if the CALIBRATING bit is still
	//set (indicating that calibration is still taking place) or cleared 
	//(indicating that the device is not in calibration mode). The calibration
	//process can take up to 10 seconds to complete; the calibration sequence
	//will time out if not sucessful within 10 seconds.
}

/* ------------------------------------------------------------ */
/*	JSTK2::calibrationDelay()
**
**	Parameters:
**		none		
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function continually checks the status register
**		for the calibrating bit to see when it is cleared.
**		Once the bit is clear, the host board will exit this routine.
*/
void JSTK2::calibrationDelay()
{
	//create a flag to check against, presuming the calibration 
	//procedure is not complete
	bool finished = false;
	while(finished == false){
		getStatus();
		if((jstkPkt.sts>>7) == 0){
			finished = true;
		}
		delayMicroseconds(100);
	}
}

/* ------------------------------------------------------------ */
/*	JSTK2::setCalXMin(uint16_t XMinCal)
**
**	Parameters:
**		uint16_t XMinCal		- user provided 10-bit calibration value
**							representing the minimum value of the X axis
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function calls the setCommand function and sends the 
**		setCalXMin command in addition to the single user provided value.
**		The 5 standard bytes will be stored in the struct variables
**		via the setCommand function.
*/
void JSTK2::setCalXMin(uint16_t XMinCal)
{
	uint8_t calValue[2];
	XMinCal &= 0x03FF;
	calValue[0] = XMinCal;
	calValue[1] = XMinCal >> 8;
	
	setCommand(cmdSetCalXMin, calValue[0], calValue[1]);
}

/* ------------------------------------------------------------ */
/*	JSTK2::setCalXMax(uint16_t XMaxCal)
**
**	Parameters:
**		uint16_t XMaxCal		- user provided 10-bit calibration value
**							representing the maximum value of the X axis
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function calls the setCommand function and sends the 
**		setCalXMax command in addition to the single user provided value.
**		The 5 standard bytes will be stored in the struct variables
**		via the setCommand function.
*/
void JSTK2::setCalXMax(uint16_t XMaxCal)
{
	uint8_t calValue[2];
	XMaxCal &= 0x03FF;
	calValue[0] = XMaxCal;
	calValue[1] = XMaxCal >> 8;
	
	setCommand(cmdSetCalXMax, calValue[0], calValue[1]);
}

/* ------------------------------------------------------------ */
/*	JSTK2::setCalYMin(uint16_t YMinCal)
**
**	Parameters:
**		uint16_t YMinCal		- user provided 10-bit calibration value
**							representing the minimum value of the Y axis
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function calls the setCommand function and sends the 
**		setCalYMin command in addition to the single user provided value.
**		The 5 standard bytes will be stored in the struct variables
**		via the setCommand function.
*/
void JSTK2::setCalYMin(uint16_t YMinCal)
{
	uint8_t calValue[2];
	YMinCal &= 0x03FF;
	calValue[0] = YMinCal;
	calValue[1] = YMinCal >> 8;
	
	setCommand(cmdSetCalYMin, calValue[0], calValue[1]);
}

/* ------------------------------------------------------------ */
/*	JSTK2::setCalYMax(uint16_t YMaxCal)
**
**	Parameters:
**		uint16_t YMaxCal		- user provided 10-bit calibration value
**							representing the maximum value of the Y axis
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function calls the setCommand function and sends the 
**		setCalYMax command in addition to the single user provided value.
**		The 5 standard bytes will be stored in the struct variables
**		via the setCommand function.
*/
void JSTK2::setCalYMax(uint16_t YMaxCal)
{
	uint8_t calValue[2];
	YMaxCal &= 0x03FF;
	calValue[0] = YMaxCal;
	calValue[1] = YMaxCal >> 8;
	
	setCommand(cmdSetCalYMax, calValue[0], calValue[1]);
}

/* ------------------------------------------------------------ */
/*	JSTK2::setCalXCenMin(uint16_t XCenMinCal)
**
**	Parameters:
**		uint16_t XCenMinCal		- user provided 10-bit calibration value
**					representing the minimum center value of the X axis
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function calls the setCommand function and sends the 
**		setCalXCenMin command in addition to the single user provided value.
**		The 5 standard bytes will be stored in the struct variables
**		via the setCommand function.
*/
void JSTK2::setCalXCenMin(uint16_t XCenMinCal)
{
	uint8_t calValue[2];
	XCenMinCal &= 0x03FF;
	calValue[0] = XCenMinCal;
	calValue[1] = XCenMinCal >> 8;
	
	setCommand(cmdSetCalXCenMin, calValue[0], calValue[1]);
}

/* ------------------------------------------------------------ */
/*	JSTK2::setCalXCenMax(uint16_t XCenMaxCal)
**
**	Parameters:
**		uint16_t XCenMaxCal		- user provided 10-bit calibration value
**					representing the maximum center value of the X axis
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function calls the setCommand function and sends the 
**		setCalXCenMax command in addition to the single user provided value.
**		The 5 standard bytes will be stored in the struct variables
**		via the setCommand function.
*/
void JSTK2::setCalXCenMax(uint16_t XCenMaxCal)
{
	uint8_t calValue[2];
	XCenMaxCal &= 0x03FF;
	calValue[0] = XCenMaxCal;
	calValue[1] = XCenMaxCal >> 8;
	
	setCommand(cmdSetCalXCenMax, calValue[0], calValue[1]);
}

/* ------------------------------------------------------------ */
/*	JSTK2::setCalYCenMin(uint16_t YCenMinCal)
**
**	Parameters:
**		uint16_t YCenMinCal		- user provided 10-bit calibration value
**					representing the minimum center value of the Y axis
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function calls the setCommand function and sends the 
**		setCalYCenMin command in addition to the single user provided value.
**		The 5 standard bytes will be stored in the struct variables
**		via the setCommand function.
*/
void JSTK2::setCalYCenMin(uint16_t YCenMinCal)
{
	uint8_t calValue[2];
	YCenMinCal &= 0x03FF;
	calValue[0] = YCenMinCal;
	calValue[1] = YCenMinCal >> 8;
	
	setCommand(cmdSetCalYCenMin, calValue[0], calValue[1]);
}

/* ------------------------------------------------------------ */
/*	JSTK2::setCalYCenMax(uint16_t YCenMaxCal)
**
**	Parameters:
**		uint16_t YCenMaxCal		- user provided 10-bit calibration value
**					representing the maximum center value of the Y axis
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function calls the setCommand function and sends the 
**		setCalYCenMax command in addition to the single user provided value.
**		The 5 standard bytes will be stored in the struct variables
**		via the setCommand function.
*/
void JSTK2::setCalYCenMax(uint16_t YCenMaxCal)
{
	uint8_t calValue[2];
	YCenMaxCal &= 0x03FF;
	calValue[0] = YCenMaxCal;
	calValue[1] = YCenMaxCal >> 8;
	
	setCommand(cmdSetCalYCenMax, calValue[0], calValue[1]);
}

/* ------------------------------------------------------------ */
/*	JSTK2::setCalXMinMax(uint16_t XMinCal, uint16_t XMaxCal)
**
**	Parameters:
**		uint16_t XMinCal		- user provided 10-bit calibration value
**							representing the minimum value of the X axis
**		uint16_t XMaxCal		- user provided 10-bit calibration value
**							representing the maximum value of the X axis
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function calls the setCommand function and sends the 
**		setCalXMinMax command in addition to the two user provided values.
**		The 5 standard bytes will be stored in the struct variables
**		via the setCommand function.
*/
void JSTK2::setCalXMinMax(uint16_t XMinCal, uint16_t XMaxCal)
{
	uint8_t calValue[4];
	XMinCal &= 0x03FF;
	XMaxCal &= 0x03FF;
	calValue[0] = XMinCal;
	calValue[1] = XMinCal >> 8;
	calValue[2] = XMaxCal;
	calValue[3] = XMaxCal >> 8;
	
	setCommand(cmdSetCalXMinMax, calValue[0], calValue[1], calValue[2], calValue[3]);
}

/* ------------------------------------------------------------ */
/*	JSTK2::setCalYMinMax(uint16_t YMinCal, uint16_t YMaxCal)
**
**	Parameters:
**		uint16_t YMinCal		- user provided 10-bit calibration value
**							representing the minimum value of the Y axis
**		uint16_t YMaxCal		- user provided 10-bit calibration value
**							representing the maximum value of the Y axis
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function calls the setCommand function and sends the 
**		setCalYMinMax command in addition to the two user provided values.
**		The 5 standard bytes will be stored in the struct variables
**		via the setCommand function.
*/
void JSTK2::setCalYMinMax(uint16_t YMinCal, uint16_t YMaxCal)
{
	uint8_t calValue[4];
	YMinCal &= 0x03FF;
	YMaxCal &= 0x03FF;
	calValue[0] = YMinCal;
	calValue[1] = YMinCal >> 8;
	calValue[2] = YMaxCal;
	calValue[3] = YMaxCal >> 8;
	
	setCommand(cmdSetCalYMinMax, calValue[0], calValue[1], calValue[2], calValue[3]);
}

/* ------------------------------------------------------------ */
/*	JSTK2::setCalXCenMinMax(uint16_t XCenMinCal, uint16_t XCenMaxCal)
**
**	Parameters:
**		uint16_t XCenMinCal		- user provided 10-bit calibration value
**					representing the minimum center value of the X axis
**		uint16_t XCenMaxCal		- user provided 10-bit calibration value
**					representing the maximum center value of the X axis
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function calls the setCommand function and sends the 
**		setCalXCenMinMax command in addition to the two user provided values.
**		The 5 standard bytes will be stored in the struct variables
**		via the setCommand function.
*/
void JSTK2::setCalXCenMinMax(uint16_t XCenMinCal, uint16_t XCenMaxCal)
{
	uint8_t calValue[4];
	XCenMinCal &= 0x03FF;
	XCenMaxCal &= 0x03FF;
	calValue[0] = XCenMinCal;
	calValue[1] = XCenMinCal >> 8;
	calValue[2] = XCenMaxCal;
	calValue[3] = XCenMaxCal >> 8;
	
	setCommand(cmdSetCalXCenMinMax, calValue[0], calValue[1], calValue[2], calValue[3]);
}

/* ------------------------------------------------------------ */
/*	JSTK2::setCalYCenMinMax(uint16_t YCenMinCal, uint16_t YCenMaxCal)
**
**	Parameters:
**		uint16_t YCenMinCal		- user provided 10-bit calibration value
**					representing the minimum center value of the Y axis
**		uint16_t YCenMaxCal		- user provided 10-bit calibration value
**					representing the maximum center value of the Y axis
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function calls the setCommand function and sends the 
**		setCalYCenMinMax command in addition to the two user provided values.
**		The 5 standard bytes will be stored in the struct variables
**		via the setCommand function.
*/
void JSTK2::setCalYCenMinMax(uint16_t YCenMinCal, uint16_t YCenMaxCal)
{
	uint8_t calValue[4];
	YCenMinCal &= 0x03FF;
	YCenMaxCal &= 0x03FF;
	calValue[0] = YCenMinCal;
	calValue[1] = YCenMinCal >> 8;
	calValue[2] = YCenMaxCal;
	calValue[3] = YCenMaxCal >> 8;
	
	setCommand(cmdSetCalYCenMinMax, calValue[0], calValue[1], calValue[2], calValue[3]);
}

/* ------------------------------------------------------------ */
/*	JSTK2::writeFlash()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function calls the setCommand function and sends the 
**		writeFlash command to the PmodJSTK2. It takes all of the
**		calibration values currently stored in RAM on the PmodJSTK2
**		and writes them to the flash memory. 
**
**		It is strongly recommended that the user first confirm via
**		the getAllCalibrationConstants() function that the current
**		calibration constants are the ones they wish to store to flash
**		memory. Once new variables are written to the flash memory,
**		there is no way to restore the original values unless the 
**		user explictly stored the original values elsewhere on their
**		host system. 
**
**		The 5 standard bytes that are provided by the PmodJSTK2 with
**		each SPI transaction will be stored in the struct variables
**		via the setCommand function.
**
**		NOTE: It is very strongly recommended that the user wait at 
**		least 5 milliseconds before performing another SPI operation
**		with the PmodJSTK2 by bringing it's chip select line low. 
**		The Pmod has interrupts disabled during	the time calibration 
**		variables are being written to the flash memory and cannot 
**		respond to any traffic on the SPI bus during this time.
*/
void JSTK2::writeFlash()
{
	//send the command
	setCommand(cmdWriteFlash);
	if(manualSpiDelay == false){
		delay(5);
	}
	//strongly recommended that the user waits 5 milliseconds
	//after the chip select line goes high in the setCommand
	//function before performing another SPI operation with this
	//device.
}

/* ------------------------------------------------------------ */
/*	JSTK2::rldFromFlash()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		This function calls the setCommand function and sends the 
**		rldFromFlash command to the PmodJSTK2. It takes all of the
**		calibration values currently stored in flash memory on the 
**		PmodJSTK2 and writes them to the RAM. 
**
**		In general, this function will only need to be called if a 
**		calibration procedure was performed on the PmodJSTK2 and it
**		was determined that the resulting values were not what the 
**		user hoped achieve and the user feels that the values that
**		were stored and loaded from the flash memory were better 
**		suited for their application.
**
**		The 5 standard bytes that are provided by the PmodJSTK2 with
**		each SPI transaction will be stored in the struct variables
**		via the setCommand function.
**
**		NOTE: It is very strongly recommended that the user wait at 
**		least 100 microseconds before performing another SPI operation
**		with the PmodJSTK2 by bringing it's chip select line low. 
**		The Pmod has interrupts disabled during	the time calibration 
**		variables are being written to the flash memory and cannot 
**		respond to any traffic on the SPI bus during this time.
*/
void JSTK2::rldFromFlash()
{
	//send the command
	setCommand(cmdRldFromFlash);
	if(manualSpiDelay == false){
		delayMicroseconds(100);
	}
	//strongly recommended that the user waits 100 microseconds
	//after the chip select line goes high in the setCoand
	//function before performing another SPI operation with this
	//device.
}

/* ------------------------------------------------------------ */
/*	JSTK2::getAllCalibrationConstants()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		Call all of the getCal* functions so that the user
**		can compare all of them simultaneously if they wish.
**
**		Force the 25 microsecond delay between function calls
**		except for the very last function call.
*/
void JSTK2::getAllCalibrationConstants()
{
	getCalXMin();
	delayMicroseconds(25);
	
	getCalXMax();
	delayMicroseconds(25);
	
	getCalYMin();
	delayMicroseconds(25);
	
	getCalYMax();
	delayMicroseconds(25);
	
	getCalXCenMin();
	delayMicroseconds(25);
	
	getCalXCenMax();
	delayMicroseconds(25);
	
	getCalYCenMin();
	delayMicroseconds(25);
	
	getCalYCenMax();
}

/* ------------------------------------------------------------ */
/*	JSTK2::setCommand()
**
**	Parameters:
**		
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		
*/
void JSTK2::setCommand(uint8_t cmd, uint8_t param1, uint8_t param2, uint8_t param3, uint8_t param4)
{
	//it's required that there is a 15 us delay between SS being
	//brought low and the transfer of the first byte
	//then 10 us delay between inter-byte transfers
	//then a 25 us before a new SPI session after SS is brought high
	
	//the first 5 bytes received in a transmission are the standard bytes. 
	//These will be collected and stored in the struct in the .h file
	
	//collect all of the command parameters into a single array
	uint8_t cmdData [5] = {cmd, param1, param2, param3, param4};
	//array to hold the standard packet values
	uint8_t stdPktValues [5];
	
	//if SPI is enabled
	if(pdspi != NULL){
		//bring the chip select line low
		digitalWrite(m_SSPin, LOW);
		//delay as appropriate
		delayMicroseconds(15);
		//send the first four pieces of the set command
		//and delay as appropriate
		for(uint8_t i=0; i<4; i++){
			stdPktValues[i] = pdspi->transfer(cmdData[i]);
			delayMicroseconds(10);
		}
		stdPktValues[4] = pdspi->transfer(cmdData[4]);
		//bring the chip select line high
		digitalWrite(m_SSPin, HIGH);
		//delay if user is not manually doing so
		if(manualSpiDelay == false){
			delayMicroseconds(25);
		}
		//otherwise, it's on the user to delay the final 25 us
		
		//we need to store the standard packet values
		//get the 10 bit X value first
		jstkPkt.smpX = ((uint16_t)stdPktValues[1] << 8) | stdPktValues[0];
		//then the 10 bit y value
		jstkPkt.smpY = ((uint16_t)stdPktValues[3] << 8) | stdPktValues[2];
		//then the state of the buttons
		jstkPkt.fsBtns = stdPktValues[4];
	}
}

/* ------------------------------------------------------------ */
/*	JSTK2::receiveCommand()
**
**	Parameters:
**		
**
**	Return Value:
**		none
**
**	Errors:
**		none
**
**	Description:
**		
*/
void JSTK2::receiveCommand(uint8_t cmd, uint8_t byteAmount, uint8_t *dataPointer)
{
	//it's required that there is a 16 us delay between SS being
	//brought low and the transfer of the first byte
	//then 10 us delay between inter-byte transfers
	//then a 25 us before a new SPI session after SS is brought high
	
	//the first 5 bytes received in a transmission are the standard bytes. 
	//These will be collected and stored in the struct in the .h file
	//byteAmount is a parameter that accounts for all of the extra
	//bytes that will be received. The extras are already handled and placed
	//into structs by the function that called receiveCommand
	
	//variable for our for loops
	int ib;
	//array to hold the standard packet values
	uint8_t stdPktValues [5];
	//variable to make Arduino IDE happy
	uint8_t blank = 0;	
	//if SPI is enabled
	if(pdspi != NULL){
		//bring chip select low
		digitalWrite(m_SSPin, LOW);
		//delay required amount of time
		delayMicroseconds(15);
		//get first standard packet value and send the command
		stdPktValues [0] = pdspi->transfer(cmd);
		//delay required amount of time
		delayMicroseconds(10);
		//get the remaining four standard packet values
		//and delay as appropriate
		for(ib = 1; ib < 5; ib++){
			stdPktValues[ib] = pdspi->transfer(blank);
			delayMicroseconds(10);
		}
		//if there are more bytes to be received, get those
		//as well and delay as appropriate
		for(ib = 0; ib < byteAmount; ib++){
			dataPointer[ib] = pdspi->transfer(blank);
			delayMicroseconds(10);
		}
		//bring the chip select high
		digitalWrite(m_SSPin, HIGH);
		//delay if user is not manually doing so
		if(manualSpiDelay == false){
			delayMicroseconds(25);
		}
		//otherwise, it's on the user to delay the final 25 us
		
		//the function that called this one already stores the extra values
		//but we need to store the standard packet values
		//get the 10 bit X value first
		jstkPkt.smpX = ((uint16_t)stdPktValues[1] << 8) | stdPktValues[0];
		//then the 10 bit y value
		jstkPkt.smpY = ((uint16_t)stdPktValues[3] << 8) | stdPktValues[2];
		//then the state of the buttons
		jstkPkt.fsBtns = stdPktValues[4];
	}
}