/************************************************************************/
/*                                                                      */
/*		PmodJSTK2.h  --	PmodJSTK2 Command and Interface Functions		*/
/*                                                                      */
/************************************************************************/
/*	Author: 	James K. Colvin										    */
/*	Copyright:	2016, Digilent, Inc.									*/
/************************************************************************/
/*  File Description:                                                   */
/*                                                                      */
/*	This file contains the declarations and type definitions used for	*/
/*	accessing the PmodJSTK2.											*/
/*                                                                      */
/************************************************************************/
/*  Revision History:													*/
/*                                                                      */
/*	04/01/2016(MichaelA): created PC based application					*/
/*	04/25/2016(JamesC):	modified PC library for a Digilent MCU			*/
/*                                                                      */
/************************************************************************/

#include <DSPI.h>

/* ------------------------------------------------------------ */
/*				Miscellaneous Declarations						*/
/* ------------------------------------------------------------ */

/* Define PmodJSTK2 SPI commands.
*/
#define mskCmd				0xFC

#define cmdSetLed			0x80
#define cmdSetLedOn			cmdSetLed | 0x01
#define cmdSetLedOff		cmdSetLed

#define cmdSetLedRGB		0x84

#define cmdSetInversion		0x88
#define cmdSetInversionX	0x88 | 0x02
#define cmdSetInversionY	0x88 | 0x01
#define cmdSetInversionOff	cmdSetInversion

#define cmdGetPosition		0xC0

#define cmdGetStatus		0xF0
#define cmdGetFirmwareVer	0xF1

#define cmdGetCalXMin		0xE0
#define cmdGetCalXMax		0xE1
#define cmdGetCalYMin		0xE2
#define cmdGetCalYMax		0xE3
#define cmdGetCalXCenMin	0xE4
#define cmdGetCalXCenMax	0xE5
#define cmdGetCalYCenMin	0xE6
#define cmdGetCalYCenMax	0xE7

#define cmdCalibrate		0xA4

#define cmdSetCalXMin		0xA8
#define cmdSetCalXMax		0xA9
#define cmdSetCalYMin		0xAA
#define cmdSetCalYMax		0xAB
#define cmdSetCalXCenMin	0xAC
#define cmdSetCalXCenMax	0xAD
#define cmdSetCalYCenMin	0xAE
#define cmdSetCalYCenMax	0xAF

#define cmdSetCalXMinMax	0xB0
#define cmdSetCalYMinMax	0xB1
#define cmdSetCalXCenMinMax	0xB2
#define cmdSetCalYCenMinMax	0xB3

#define cmdWriteFlash		0xB8
#define cmdRldFromFlash		0xBC

#define cbSndMax			16

/* Define the bit positions used in a status packet.
*/
#define bnCalibrating           7
#define bnLastCal               6
#define bnLastFWS               5
#define bnLastFRS               4
#define bnInvertX               1
#define bnInvertY               0

#define bitCalibrating          (1 << bnCalibrating)
#define bitLastCal              (1 << bnLastCal)
#define bitLastFWS              (1 << bnLastFWS)
#define bitLastFRS              (1 << bnLastFRS)
#define bitInvertX              (1 << bnInvertX)
#define bitInvertY              (1 << bnInvertY)

/* Define bit postiions used in fsButtons.
*/
#define bnExtpkt                7
#define bnTrigger               1
#define bnJstk                  0

#define bitExtpkt               (1 << bnExtpkt)
#define bitTrigger              (1 << bnTrigger)
#define bitJstk                 (1 << bnJstk)

/** defines that need to be supported* */
/*	Led States
*/
#define JSTK_LED1 0x01
#define JSTK_LED2 0x02

/*  Button States
*/
#define JSTK_BTN1 0x02
#define JSTK_BTN2 0x04

/* ------------------------------------------------------------ */
/*				Type Declarations								*/
/* ------------------------------------------------------------ */

/* Define a data structure that can be used to return a Joystick data packet.
*/

struct JSTKPKT {
	uint16_t smpX;
	uint16_t smpY;
	uint8_t fsBtns;
	/*	now list the other possible variables		*/
	/*	that could be collected in the data packet	*/
	/*	received from the PmodJSTK2					*/
	
	//8-bit positions
	uint8_t posX;
	uint8_t posY;
	
	//calibration variable
	uint16_t calvar;
	
	//status
	uint8_t sts;
	
	//firmware version
	uint16_t fwver;
};

struct JSTKCMD {
	uint8_t cmd;

	//up to four parameters could be provided
	uint8_t param1;
	uint8_t param2;
	uint8_t param3;
	uint8_t param4;
};

struct JSTKCAL {
	uint16_t smpXMin;	//factory loaded value is 127
	uint16_t smpXMax;	//factory loaded value is 824
	uint16_t smpYMin;	//factory loaded value is 159
	uint16_t smpYMax;	//factory loaded value is 895
	uint16_t smpXCenterMin;	//factory loaded value is 421
	uint16_t smpXCenterMax;	//factory loaded value is 521
	uint16_t smpYCenterMin;	//factory loaded value is 473
	uint16_t smpYCenterMax;	//factory loaded value is 573
};

/* ------------------------------------------------------------ */
/*					Object Class Declarations					*/
/* ------------------------------------------------------------ */

class JSTK2
{
	public:
		JSTK2();
		void begin(int SpiPort = 0, bool userDelay = false);
		void end();
		
		void setLedOn();	//done
		void setLedOff();	//done
		void setLedRGB(uint8_t red, uint8_t green, uint8_t blue);	//done
		void setInversionXY(uint8_t xInversion, uint8_t yInversion);	//done
		void setInversionX(uint8_t xInversion);	//done
		void setInversionY(uint8_t yInversion);	//done
		void setInversionOff();	//done
		void getPosition();	//done
		void getStatus();	//done
		void getFirmwareVer();	//done
		void getCalXMin();	//done
		void getCalXMax();	//done
		void getCalYMin();	//done
		void getCalYMax();	//done
		void getCalXCenMin();	//done
		void getCalXCenMax();	//done
		void getCalYCenMin();	//done
		void getCalYCenMax();	//done
		void calibrate();	//done
		void setCalXMin(uint16_t XMinCal);	//done
		void setCalXMax(uint16_t XMaxCal);	//done
		void setCalYMin(uint16_t YMinCal);	//done
		void setCalYMax(uint16_t YMaxCal);	//done
		void setCalXCenMin(uint16_t XCenMinCal);	//done
		void setCalXCenMax(uint16_t XCenMaxCal);	//done
		void setCalYCenMin(uint16_t YCenMinCal);	//done
		void setCalYCenMax(uint16_t YCenMaxCal);	//done
		void setCalXMinMax(uint16_t XMinCal, uint16_t XMaxCal);	//done
		void setCalYMinMax(uint16_t YMinCal, uint16_t YMaxCal);	//done
		void setCalXCenMinMax(uint16_t XCenMinCal, uint16_t XCenMaxCal);	//done
		void setCalYCenMinMax(uint16_t YCenMinCal, uint16_t YCenMaxCal);	//done
		void writeFlash();	//done
		void rldFromFlash();	//done
		
		//extra function added by me
		void getAllCalibrationConstants();	//done
		void calibrationDelay();	//done
		void setInversion(uint8_t xInversion, uint8_t yInversion);	//done
		
		//declaring the structs (I think that's termed correctly)
		struct JSTKPKT jstkPkt;
		struct JSTKCMD jstkCmd;
		struct JSTKCAL jstkCal;
		
		//test function to determine if SPI is read correctly
		

	//Legacy JSTK functions that need to be supported
		void setLeds(uint8_t ledSt); //accounted for
		uint16_t getX();	//accounted for
		uint16_t getY();	//accounted for
		uint8_t getBtn();	//accounted for
		void getAll(uint8_t ledSt, uint8_t *recv);	//accounted for

	private:
		DSPI *pdspi;
		uint8_t m_SSPin;
		void setCommand(uint8_t cmd, uint8_t param1 = 0, uint8_t param2 = 0, uint8_t param3 = 0, uint8_t param4 = 0);
		void receiveCommand(uint8_t cmd, uint8_t length, uint8_t *dataStorage);
		void JSTK2HostInit();
		void JSTK2HostTerm();
		bool manualSpiDelay;
};

/* ------------------------------------------------------------ */
/*				Interface Procedure Declarations				*/
/* ------------------------------------------------------------ */

/*
BOOL	FGetJstk(HIF hifJstk, JSTKCMD* pjstkcmdSnd, JSTKPKT* pjstkpktRcv, uint8_t cbSnd);
BOOL	FGetJstkCAL(HIF hifJstk, JSTKCAL* pjstkcalRcv);
BOOL	FGetJstkFWVER(HIF hifJstk, uint16_t* pfwverRcv);
BOOL	FCalibrateJstk(HIF hifJstk); // */