/************************************************************************/
/*                                                                      */
/*  PmodJSTK2FullDemo.ino -- Full Example Sketch for PmodJSTK2          */
/*                                                                      */
/************************************************************************/
/*  Author: James Colvin                                                */
/*  Copyright (c) 2016, Digilent                                        */
/************************************************************************/
/*
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.
*/
/************************************************************************/
/*  Module Description:                                                 */
/*                                                                      */
/*  This sketch shows off all of the functions available in the         */
/*  PmodJSTK2 library and implements the manual user delays after       */
/*  each SPI transaction.                                               */
/*                                                                      */
/*  Note that the embedded PIC16 only updates the values for the two    */
/*  axes at a rate of 100 Hz or every 10 milliseconds.                  */
/*                                                                      */
/************************************************************************/
/*  Revision History:                                                   */
/*                                                                      */
/*  6/02/2016(James C): Created                                         */
/*                                                                      */
/************************************************************************/
/************************************************************************/
/*  Board Support:                                                      */
/*                                                                      */
/*  chipKIT WF32:               DSPI0 with SS as pin 10 (not pin 44)    */
/*                       MOSI - 11, MISO - 12, and SCLK - 13 as normal  */
/*  chipKIT Uno32, uC32, WiFIRE  DSPI0 (pins 10, 11, 12, 13 are default)*/
/*                                                                      */
/************************************************************************/

#include <DSPI.h>
#include <PmodJSTK2.h>
JSTK2 jstk2;

void setup(){
  //start up the serial monitor
  Serial.begin(9600);
  delay(10);
  //initialize the JSTK2 class, DSPI0, and have it be
  //on the user to implement delays after the CS
  //line goes high: 
  //5 milliseconds after writing to the flash memory
  //100 microseconds after reading from the flash memory
  //25 microseconds for all other functions
  jstk2.begin(0, true);
  delay(1000);
  
  Serial.println();Serial.println("supporting the legacy PmodJSTK functions");
  jstk2.setLeds(1);
  delayMicroseconds(25);
  uint16_t xAxis = jstk2.getX();
  delayMicroseconds(25);
  Serial.print("10-bit X axis value is: ");Serial.println(xAxis, DEC);
  uint16_t yAxis = jstk2.getY();
  delayMicroseconds(25);
  Serial.print("10-bit Y axis value is: ");Serial.println(yAxis, DEC);
  uint8_t btns = jstk2.getBtn();
  delayMicroseconds(25);
  Serial.print("8-bit button state: ");Serial.println(btns, DEC);
  
  Serial.println();Serial.println("showing off the all of the new functions");  
  
  Serial.println();Serial.println("setLedRGB function");
  jstk2.setLedRGB(240, 55, 180);
  delayMicroseconds(25);
  Serial.println("every time any function is called, 5 bytes are collected from the PmodJSTK2");
  Serial.println("these bytes are the 10-bit X and Y axis values, and the state of the buttons");
  Serial.print("10-bit X value: ");Serial.println(jstk2.jstkPkt.smpX);
  Serial.print("10-bit Y value: ");Serial.println(jstk2.jstkPkt.smpY);
  Serial.print("state of the buttons: ");Serial.println(jstk2.jstkPkt.fsBtns, BIN);
  Serial.println("Here's a list of what each bit means for the button state");
  Serial.print("bit 7 (MSB): ");Serial.println("SPI packet extension, set if command supports it, cleared if not -- cleared on powerup");
  Serial.print("bits 2 through 6: ");Serial.println("not used -- cleared on powerup");
  Serial.print("bit 1: ");Serial.println("trigger button, set if pressed, cleared if not -- cleared on powerup");
  Serial.print("bit 0 (LSB): ");Serial.println("joystick shaft button, set if pressed, cleared if not -- cleared on powerup");
  
  Serial.println();Serial.println("8-bit position and inversion functions");
  Serial.println("move the joystick away from the center position");
  delay(2500); //to allow the user time to move the joystick away from the center position 
  jstk2.getPosition();
  delayMicroseconds(25);
  Serial.println("the MSB of the button state byte will be set if a command");
  Serial.println("requesting data in additiona to the standard 5 bytes was successful");
  Serial.print("state of the buttons: ");Serial.println(jstk2.jstkPkt.fsBtns, BIN);
  Serial.print("8-bit X axis position: ");Serial.println(jstk2.jstkPkt.posX, DEC);
  Serial.print("8-bit Y axis position: ");Serial.println(jstk2.jstkPkt.posY, DEC);
  Serial.println("invert the results of the 8-bit position values and remeasure");
  jstk2.setInversion(1, 1);
  delayMicroseconds(25);
  //Delay for an additional 10 milliseconds because the PmodJSTK2 only collects new
  //joystick data (and thus 8-bit positional data) at a rate of 100 Hz
  delay(10);
  jstk2.getPosition();
  delayMicroseconds(25);
  Serial.print("8-bit X axis position: ");Serial.println(jstk2.jstkPkt.posX, DEC);
  Serial.print("8-bit Y axis position: ");Serial.println(jstk2.jstkPkt.posY, DEC);
  Serial.println("turn off the inversion for both axes and remeasure");
  jstk2.setInversionOff();
  delayMicroseconds(25);
  //Delay for an additional 10 milliseconds because the PmodJSTK2 only collects new
  //joystick data (and thus 8-bit positional data) at a rate of 100 Hz
  delay(10);
  jstk2.getPosition();
  delayMicroseconds(25);
  Serial.print("8-bit X axis position: ");Serial.println(jstk2.jstkPkt.posX, DEC);
  Serial.print("8-bit Y axis position: ");Serial.println(jstk2.jstkPkt.posY, DEC);
  
  Serial.println();Serial.println("get the firmware version of the software embedded on the PIC16");
  jstk2.getFirmwareVer();
  delayMicroseconds(25);
  Serial.print("The firmware version on the module is: ");
  Serial.print(jstk2.jstkPkt.fwver>>8, DEC);Serial.print(".");Serial.println(uint8_t(jstk2.jstkPkt.fwver), DEC);
  
  Serial.println();Serial.println("get the status of the PmodJSTK2");
  jstk2.getStatus();
  delayMicroseconds(25);
  Serial.print("8-bit status value: ");Serial.println(jstk2.jstkPkt.sts, BIN);
  Serial.println("Here's a list of what each bit means");
  Serial.print("bit 7 (MSB): ");Serial.println("calibrating bit, set if in calibration routine, cleared if not -- cleared on powerup");
  Serial.print("bit 6: ");Serial.println("Last Calibration routine successful bit, set if true, cleared if false -- set on powerup");
  Serial.print("bit 5: ");Serial.println("Last Flash Write successful bit, set if true, cleared if false -- cleared on powerup");
  Serial.print("bit 4: ");Serial.println("Last Flash Read successful bit, set if true, cleared if false -- set on powerup since calibration values are read from flash into upon powerup");
  Serial.print("bit 3: ");Serial.println("not used -- cleared on powerup");
  Serial.print("bit 2: ");Serial.println("not used -- cleared on powerup");
  Serial.print("bit 1: ");Serial.println("8-bit X position inverted, set if true, cleared if false -- cleared on powerup");
  Serial.print("bit 0 (LSB): ");Serial.println("8-bit Y position inverted, set if true, cleared if false -- cleared on powerup");
  
  
  Serial.println();Serial.println("showing off all of the commands related to calibration");
  //reload calibration values from flash in case users run this sketch
  //multiple times without power-cycling the system board in order to 
  //make sure that the original collected values are correct and not 
  //the RAM values from the previous run-through.
  Serial.println();Serial.println("Reloading of calibration values from Flash memory");Serial.println();
  jstk2.rldFromFlash();
  delayMicroseconds(100);
  
  //collect all of the calibration constants (loaded from flash upon power-up)
  //and print them out
  jstk2.getAllCalibrationConstants();
  delayMicroseconds(25);
  Serial.print("power-up smpXMin: ");Serial.println(jstk2.jstkCal.smpXMin, DEC);
  Serial.print("power-up smpXMax: ");Serial.println(jstk2.jstkCal.smpXMax, DEC);
  Serial.print("power-up smpYMin: ");Serial.println(jstk2.jstkCal.smpYMin, DEC);
  Serial.print("power-up smpYMax: ");Serial.println(jstk2.jstkCal.smpYMax, DEC);
  Serial.print("power-up smpXCenterMin: ");Serial.println(jstk2.jstkCal.smpXCenterMin, DEC);
  Serial.print("power-up smpXCenterMax: ");Serial.println(jstk2.jstkCal.smpXCenterMax, DEC);
  Serial.print("power-up smpYCenterMin: ");Serial.println(jstk2.jstkCal.smpYCenterMin, DEC);
  Serial.print("power-up smpYCenterMax: ");Serial.println(jstk2.jstkCal.smpYCenterMax, DEC);
  //Store all of these collected calibration values to compare against later
  int XMin = jstk2.jstkCal.smpXMin;
  int XMax = jstk2.jstkCal.smpXMax;
  int YMin = jstk2.jstkCal.smpYMin;
  int YMax = jstk2.jstkCal.smpYMax;
  int XCenterMin = jstk2.jstkCal.smpXCenterMin;
  int XCenterMax = jstk2.jstkCal.smpXCenterMax;
  int YCenterMin = jstk2.jstkCal.smpYCenterMin;
  int YCenterMax = jstk2.jstkCal.smpYCenterMax;
  
  Serial.println();
  Serial.println("calibrating, rotate the Joystick around in all directions");
  /*
  If the joystick has been rotated around and then detected to be at rest
  for a full second within 10 seconds time, the blue LED will stop flashing
  and the green LED will flash twice indicating that calibration was succesful.
  If neither of those conditions are met within 10 seconds, the blue LED will
  stop flashing, and the red LED will flash twice indicating that the 
  calibration sequence was unsuccessful.
  */ 
  jstk2.calibrate();
  delayMicroseconds(25);
  //reads the status register to see if the calibration routine is complete
  //remains inside the calibrationDelay function if it is not complete yet
  jstk2.calibrationDelay();
  Serial.println("calibration complete");Serial.println();
  
  //re-collect all of the calibration constants and print them out
  jstk2.getAllCalibrationConstants();
  delayMicroseconds(25);
  Serial.print("new smpXMin: ");Serial.println(jstk2.jstkCal.smpXMin, DEC);
  Serial.print("new smpXMax: ");Serial.println(jstk2.jstkCal.smpXMax, DEC);
  Serial.print("new smpYMin: ");Serial.println(jstk2.jstkCal.smpYMin, DEC);
  Serial.print("new smpYMax: ");Serial.println(jstk2.jstkCal.smpYMax, DEC);
  Serial.print("new smpXCenterMin: ");Serial.println(jstk2.jstkCal.smpXCenterMin, DEC);
  Serial.print("new smpXCenterMax: ");Serial.println(jstk2.jstkCal.smpXCenterMax, DEC);
  Serial.print("new smpYCenterMin: ");Serial.println(jstk2.jstkCal.smpYCenterMin, DEC);
  Serial.print("new smpYCenterMax: ");Serial.println(jstk2.jstkCal.smpYCenterMax, DEC);
  
  //reload all of the original calibration values from the PmodJSTK2's flash memory
  Serial.println();Serial.println("reload calibration values from flash");
  jstk2.rldFromFlash();
  delayMicroseconds(100);
  
  //re-collect all of the calibration constants and print them out, comparing them to
  //the original values that were collected
  jstk2.getAllCalibrationConstants();
  delayMicroseconds(25);
  Serial.println();
  Serial.print("flash smpXMin: ");Serial.print(jstk2.jstkCal.smpXMin, DEC);Serial.print(", original smpXMin: ");Serial.println(jstk2.jstkCal.smpXMin, DEC);
  Serial.print("flash smpXMax: ");Serial.print(jstk2.jstkCal.smpXMax, DEC);Serial.print(", original smpXMax: ");Serial.println(jstk2.jstkCal.smpXMax, DEC);
  Serial.print("flash smpYMin: ");Serial.print(jstk2.jstkCal.smpYMin, DEC);Serial.print(", original smpYMin: ");Serial.println(jstk2.jstkCal.smpYMin, DEC);
  Serial.print("flash smpYMax: ");Serial.print(jstk2.jstkCal.smpYMax, DEC);Serial.print(", original smpYMax: ");Serial.println(jstk2.jstkCal.smpYMax, DEC);
  Serial.print("flash smpXCenterMin: ");Serial.print(jstk2.jstkCal.smpXCenterMin, DEC);Serial.print(", original smpXCenterMin: ");Serial.println(jstk2.jstkCal.smpXCenterMin, DEC);
  Serial.print("flash smpXCenterMax: ");Serial.print(jstk2.jstkCal.smpXCenterMax, DEC);Serial.print(", original smpXCenterMax: ");Serial.println(jstk2.jstkCal.smpXCenterMax, DEC);
  Serial.print("flash smpYCenterMin: ");Serial.print(jstk2.jstkCal.smpYCenterMin, DEC);Serial.print(", original smpYCenterMin: ");Serial.println(jstk2.jstkCal.smpYCenterMin, DEC);
  Serial.print("flash smpYCenterMax: ");Serial.print(jstk2.jstkCal.smpYCenterMax, DEC);Serial.print(", original smpYCenterMax: ");Serial.println(jstk2.jstkCal.smpYCenterMax, DEC);
  
  //Now we will load specific calibration values (10-bit) into RAM
  /*
  Note that provided parameters must meet certain criteria in order to be 
  //successfully loaded into the RAM of the embedded PIC16
  //For example, a provided 10-bit calibration value for the minimum Y 
  //axis value must be between an internally set Y minimum value and an 
  //internally set Y Center minimum value in order to be successfully set.
  */
  Serial.println();
  jstk2.setCalXMin(XMin+5);
  delayMicroseconds(25);
  jstk2.setCalXMax(XMax-5);
  delayMicroseconds(25);
  jstk2.setCalYMin(YMin+5);
  delayMicroseconds(25);
  jstk2.setCalYMax(YMax-5);
  delayMicroseconds(25);
  jstk2.setCalXCenMin(XCenterMin+5);
  delayMicroseconds(25);
  jstk2.setCalXCenMax(XCenterMax-5);
  delayMicroseconds(25);
  jstk2.setCalYCenMin(YCenterMin+5);
  delayMicroseconds(25);
  jstk2.setCalYCenMax(YCenterMax-5);
  delayMicroseconds(25);
  
  //re-collect all of the calibration constants and print them out, comparing them to
  //the original values that were collected
  jstk2.getAllCalibrationConstants();
  delayMicroseconds(25);
  Serial.print("smpXMin+5: ");Serial.print(jstk2.jstkCal.smpXMin, DEC);Serial.print(", original smpXMin: ");Serial.println(XMin, DEC);
  Serial.print("smpXMax-5: ");Serial.print(jstk2.jstkCal.smpXMax, DEC);Serial.print(", original smpXMax: ");Serial.println(XMax, DEC);
  Serial.print("smpYMin+5: ");Serial.print(jstk2.jstkCal.smpYMin, DEC);Serial.print(", original smpYMin: ");Serial.println(YMin, DEC);
  Serial.print("smpYMax-5: ");Serial.print(jstk2.jstkCal.smpYMax, DEC);Serial.print(", original smpYMax: ");Serial.println(YMax, DEC);
  Serial.print("smpXCenterMin+5: ");Serial.print(jstk2.jstkCal.smpXCenterMin, DEC);Serial.print(", original smpXCenterMin: ");Serial.println(XCenterMin, DEC);
  Serial.print("smpXCenterMax-5: ");Serial.print(jstk2.jstkCal.smpXCenterMax, DEC);Serial.print(", original smpXCenterMax: ");Serial.println(XCenterMax, DEC);
  Serial.print("smpYCenterMin+5: ");Serial.print(jstk2.jstkCal.smpYCenterMin, DEC);Serial.print(", original smpYCenterMin: ");Serial.println(YCenterMin, DEC);
  Serial.print("smpYCenterMax-5: ");Serial.print(jstk2.jstkCal.smpYCenterMax, DEC);Serial.print(", original smpYCenterMax: ");Serial.println(YCenterMax, DEC);
  
  //Now we will load specific calibration values (10-bit) into RAM
  //but this time with alternate functions, just for fun.
  /*
  Note that provided parameters must meet certain criteria in order to be 
  //successfully loaded into the RAM of the embedded PIC16
  //For example, a provided 10-bit calibration value for the minimum Y 
  //axis value must be between an internally set Y minimum value and an 
  //internally set Y Center minimum value in order to be successfully set.
  */
  Serial.println();
  jstk2.setCalXMinMax(XMin+10, XMax-10);
  delayMicroseconds(25);
  jstk2.setCalYMinMax(YMin+10, YMax-10);
  delayMicroseconds(25);
  jstk2.setCalXCenMinMax(XCenterMin+10, XCenterMax-10);
  delayMicroseconds(25);
  jstk2.setCalYCenMinMax(YCenterMin+10, YCenterMax-10);
  delayMicroseconds(25);
  
  //re-collect all of the calibration constants and print them out, comparing them to
  //the original values that were collected
  jstk2.getAllCalibrationConstants();
  delayMicroseconds(25);
  Serial.print("new smpXMin+10: ");Serial.print(jstk2.jstkCal.smpXMin, DEC);Serial.print(", original smpXMin: ");Serial.println(XMin, DEC);
  Serial.print("new smpXMax-10: ");Serial.print(jstk2.jstkCal.smpXMax, DEC);Serial.print(", original smpXMax: ");Serial.println(XMax, DEC);
  Serial.print("new smpYMin+10: ");Serial.print(jstk2.jstkCal.smpYMin, DEC);Serial.print(", original smpYMin: ");Serial.println(YMin, DEC);
  Serial.print("new smpYMax-10: ");Serial.print(jstk2.jstkCal.smpYMax, DEC);Serial.print(", original smpYMax: ");Serial.println(YMax, DEC);
  Serial.print("new smpXCenterMin+10: ");Serial.print(jstk2.jstkCal.smpXCenterMin, DEC);Serial.print(", original smpXCenterMin: ");Serial.println(XCenterMin, DEC);
  Serial.print("new smpXCenterMax-10: ");Serial.print(jstk2.jstkCal.smpXCenterMax, DEC);Serial.print(", original smpXCenterMax: ");Serial.println(XCenterMax, DEC);
  Serial.print("new smpYCenterMin+10: ");Serial.print(jstk2.jstkCal.smpYCenterMin, DEC);Serial.print(", original smpYCenterMin: ");Serial.println(YCenterMin, DEC);
  Serial.print("new smpYCenterMax-10: ");Serial.print(jstk2.jstkCal.smpYCenterMax, DEC);Serial.print(", original smpYCenterMax: ");Serial.println(YCenterMax, DEC);
  
  Serial.println();Serial.println("Writing the current calibration values in RAM to the embedded Flash memory");
  jstk2.writeFlash();
  delay(5);
  
  Serial.println();Serial.println("confirm values written into flash are accurate");
  jstk2.rldFromFlash();
  delayMicroseconds(100);
  //re-collect all of the calibration constants and print them out
  jstk2.getAllCalibrationConstants();
  delayMicroseconds(25);
  Serial.print("flash smpXMin: ");Serial.println(jstk2.jstkCal.smpXMin, DEC);
  Serial.print("flash smpXMax: ");Serial.println(jstk2.jstkCal.smpXMax, DEC);
  Serial.print("flash smpYMin: ");Serial.println(jstk2.jstkCal.smpYMin, DEC);
  Serial.print("flash smpYMax: ");Serial.println(jstk2.jstkCal.smpYMax, DEC);
  Serial.print("flash smpXCenterMin: ");Serial.println(jstk2.jstkCal.smpXCenterMin, DEC);
  Serial.print("flash smpXCenterMax: ");Serial.println(jstk2.jstkCal.smpXCenterMax, DEC);
  Serial.print("flash smpYCenterMin: ");Serial.println(jstk2.jstkCal.smpYCenterMin, DEC);
  Serial.print("flash smpYCenterMax: ");Serial.println(jstk2.jstkCal.smpYCenterMax, DEC);
  
  Serial.println();Serial.println("Writing other values to RAM");
  jstk2.setCalXMin(XMin+5);
  delayMicroseconds(25);
  jstk2.setCalXMax(XMax-5);
  delayMicroseconds(25);
  jstk2.setCalYMin(YMin+5);
  delayMicroseconds(25);
  jstk2.setCalYMax(YMax-5);
  delayMicroseconds(25);
  jstk2.setCalXCenMin(XCenterMin+5);
  delayMicroseconds(25);
  jstk2.setCalXCenMax(XCenterMax-5);
  delayMicroseconds(25);
  jstk2.setCalYCenMin(YCenterMin+5);
  delayMicroseconds(25);
  jstk2.setCalYCenMax(YCenterMax-5);
  delayMicroseconds(25);
  
  Serial.println();Serial.println("measuring loaded calibration constants and comparing them to the original, not what's currently in flash");
  Serial.println("this is to confirm that it worked");
  jstk2.getAllCalibrationConstants();
  delayMicroseconds(25);
  Serial.print("smpXMin+5: ");Serial.print(jstk2.jstkCal.smpXMin, DEC);Serial.print(", original smpXMin: ");Serial.println(XMin, DEC);
  Serial.print("smpXMax-5: ");Serial.print(jstk2.jstkCal.smpXMax, DEC);Serial.print(", original smpXMax: ");Serial.println(XMax, DEC);
  Serial.print("smpYMin+5: ");Serial.print(jstk2.jstkCal.smpYMin, DEC);Serial.print(", original smpYMin: ");Serial.println(YMin, DEC);
  Serial.print("smpYMax-5: ");Serial.print(jstk2.jstkCal.smpYMax, DEC);Serial.print(", original smpYMax: ");Serial.println(YMax, DEC);
  Serial.print("smpXCenterMin+5: ");Serial.print(jstk2.jstkCal.smpXCenterMin, DEC);Serial.print(", original smpXCenterMin: ");Serial.println(XCenterMin, DEC);
  Serial.print("smpXCenterMax-5: ");Serial.print(jstk2.jstkCal.smpXCenterMax, DEC);Serial.print(", original smpXCenterMax: ");Serial.println(XCenterMax, DEC);
  Serial.print("smpYCenterMin+5: ");Serial.print(jstk2.jstkCal.smpYCenterMin, DEC);Serial.print(", original smpYCenterMin: ");Serial.println(YCenterMin, DEC);
  Serial.print("smpYCenterMax-5: ");Serial.print(jstk2.jstkCal.smpYCenterMax, DEC);Serial.print(", original smpYCenterMax: ");Serial.println(YCenterMax, DEC);
  
  //reload all of the calibration values from the PmodJSTK2's flash memory
  Serial.println();Serial.println("reload calibration values from flash");
  jstk2.rldFromFlash();
  delayMicroseconds(100);
  
  //re-collect all of the calibration constants and print them out, comparing them to
  //the original values that were collected
  jstk2.getAllCalibrationConstants();
  delayMicroseconds(25);
  Serial.println();
  Serial.print("flash smpXMin: ");Serial.print(jstk2.jstkCal.smpXMin, DEC);Serial.print(", original smpXMin: ");Serial.println(XMin, DEC);
  Serial.print("flash smpXMax: ");Serial.print(jstk2.jstkCal.smpXMax, DEC);Serial.print(", original smpXMax: ");Serial.println(XMax, DEC);
  Serial.print("flash smpYMin: ");Serial.print(jstk2.jstkCal.smpYMin, DEC);Serial.print(", original smpYMin: ");Serial.println(YMin, DEC);
  Serial.print("flash smpYMax: ");Serial.print(jstk2.jstkCal.smpYMax, DEC);Serial.print(", original smpYMax: ");Serial.println(YMax, DEC);
  Serial.print("flash smpXCenterMin: ");Serial.print(jstk2.jstkCal.smpXCenterMin, DEC);Serial.print(", original smpXCenterMin: ");Serial.println(XCenterMin, DEC);
  Serial.print("flash smpXCenterMax: ");Serial.print(jstk2.jstkCal.smpXCenterMax, DEC);Serial.print(", original smpXCenterMax: ");Serial.println(XCenterMax, DEC);
  Serial.print("flash smpYCenterMin: ");Serial.print(jstk2.jstkCal.smpYCenterMin, DEC);Serial.print(", original smpYCenterMin: ");Serial.println(YCenterMin, DEC);
  Serial.print("flash smpYCenterMax: ");Serial.print(jstk2.jstkCal.smpYCenterMax, DEC);Serial.print(", original smpYCenterMax: ");Serial.println(YCenterMax, DEC);
  
  Serial.println();Serial.println("Load the factory flash values back into RAM, and then back into flash");
  jstk2.setCalXMinMax(XMin, XMax);
  delayMicroseconds(25);
  jstk2.setCalYMinMax(YMin, YMax);
  delayMicroseconds(25);
  jstk2.setCalXCenMinMax(XCenterMin, XCenterMax);
  delayMicroseconds(25);
  jstk2.setCalYCenMinMax(YCenterMin, YCenterMax);
  delayMicroseconds(25);
  
  jstk2.writeFlash();
  delay(5);
  
  Serial.println();Serial.println("confirm we're all back to normal");
  //re-collect all of the calibration constants and confirm it's now correct
  jstk2.getAllCalibrationConstants();
  delayMicroseconds(25);
  Serial.print("new smpXMin: ");Serial.println(jstk2.jstkCal.smpXMin, DEC);
  Serial.print("new smpXMax: ");Serial.println(jstk2.jstkCal.smpXMax, DEC);
  Serial.print("new smpYMin: ");Serial.println(jstk2.jstkCal.smpYMin, DEC);
  Serial.print("new smpYMax: ");Serial.println(jstk2.jstkCal.smpYMax, DEC);
  Serial.print("new smpXCenterMin: ");Serial.println(jstk2.jstkCal.smpXCenterMin, DEC);
  Serial.print("new smpXCenterMax: ");Serial.println(jstk2.jstkCal.smpXCenterMax, DEC);
  Serial.print("new smpYCenterMin: ");Serial.println(jstk2.jstkCal.smpYCenterMin, DEC);
  Serial.print("new smpYCenterMax: ");Serial.println(jstk2.jstkCal.smpYCenterMax, DEC);
  Serial.println();Serial.println("Full JSTK2 Demo complete");
 
}//END OF SETUP

void loop(){
  //do nothing
}
